/**
  ******************************************************************************
  * @file     LIN.c
  * @author   sinomcu.com-AE
  * @brief    LIN PID and checksum function.
  *    
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */

#include "ms32f0xx.h"
#include "LIN.h"
#include "LIN_user.h"

/**
  * @brief  caculation PID
  * PID6bit ID+2bit check:
  *     bit6:P0 = ID0ID1ID2ID4  
  *     bit7:P1 = ~(ID3ID4ID ID1) 
  * @param  uint8_t ID 
  * @retval uint8_t PID 
  */ 
uint8_t LIN_CaclPID(uint8_t id) {
  uint8_t pid=0,P0=0,P1=0;
  
  P0 = (((id)^(id>>1)^(id>>2)^(id>>4)) & 0x01) <<6;
  P1 = ((~((id>>1)^(id>>3)^(id>>4)^(id>>5))) & 0x01) <<7;  
  pid = id | P0 | P1;
  
  return pid;
}

/**
  * @brief  caculation chacksum,data len 8
  * Diagnostic Frame,ID:0x3C,0x3D;stand checksum.Other Frame enhanced checksum
  * @param  uint8_t* buff(recv answer data) 
  * @retval uint8_t chechsum 
  */
uint8_t LIN_CaclChecksum(uint8_t pid,uint8_t *buff) {
  uint8_t count=0;
  uint8_t id=0; 
  uint16_t checksum =0;

  id = GET_ID(pid);
  /*caculation data check sum*/
  if( (id==0x3C) ||(id==0x3D) ) {
    checksum =0;        /*stand checksum*/
  }else{
    checksum =pid;      /*enhanced checksum*/ 
  }
  for(count=0;count <LIN_FRAM_DATA_LEN;count++) {
    checksum += buff[count];
    if(checksum  & 0xFF00) {
      checksum  &= 0x00FF;
      checksum  +=1;
    }
  }
  checksum = ~checksum;  

  return (uint8_t)checksum;
}

/**
  * @brief  Compare Checksum
  * @param  uint8_t* recv_buff(recv_buff[0] PID) 
  * @retval FlagStatus{RESET,SET}
  * PID correct return SET
  * PID check not ok return RESET
  */ 
FlagStatus LIN_CompareChecksum(uint8_t *buff) {
  uint8_t checksum =0;
  FlagStatus flag;

  checksum = LIN_CaclChecksum(buff[0],&buff[1]);
  if(checksum == buff[LIN_CHECKSUM_POS] ) {
    flag =SET;
  }
  else {
    flag =RESET;        /*check sum error*/
  }

  return flag;
}

/******************************** END OF FILE *********************************/